﻿using System;
using System.Collections.ObjectModel;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;

namespace TaskCancellationExample.ViewModels
{
  public class MainViewModel : ViewModelBase
  {
    private CancellationTokenSource _cancellationTokenSource = new CancellationTokenSource();
    private Task _backgroundTask = Task.CompletedTask;
    private double _progress = 0.0;

    public MainViewModel()
    {
      for (int i = 0; i < 20; i++)
      {
        Items.Add(new FileViewModel("File " + (i + 1)));
      }

      LoadCommand = new ActionCommand(LoadExecute);
    }

    public ObservableCollection<FileViewModel> Items { get; } = new ObservableCollection<FileViewModel>();

    public double Progress
    {
      get => _progress;
      set
      {
        _progress = value;
        OnPropertyChanged(nameof(Progress));
      }
    }

    public ICommand LoadCommand { get; }

    private void LoadExecute()
    {
      // Cancel the previous task
      _cancellationTokenSource.Cancel();

      try
      {
        // Wait until task is completed;
        _backgroundTask.Wait();
      }
      catch (AggregateException aggregateException)
      {
        // Ignore AggregateExceptions
        aggregateException.Handle(x => true);
      }

      // Dispose the existing token source and create a new one.
      _cancellationTokenSource.Dispose();
      _cancellationTokenSource = new CancellationTokenSource();
      CancellationToken cancellationToken = _cancellationTokenSource.Token;
      int progressDelta = 100 / Items.Count;

      _backgroundTask = Task.Run(() =>
        {
          Progress = 0;

          for (int i = 0; i < Items.Count; i++)
          {
            FileViewModel item = Items[i];
            if (!cancellationToken.IsCancellationRequested)
            {
              item.Load();
            }
            Progress += progressDelta;
          }
        }, cancellationToken);
    }
  }
}
